"""
        Yasuharu Okamoto, 2018

"""
import pystan
from pystan import StanModel
import pickle
import random
import math
import matplotlib.pyplot as plt

def icc(t, a, b, c):
    return 1.0 / (1.0 + math.exp(-1.7 * a * (t - b - c)))

def CatResponse(t, a, b, c, K):
    res = 1
    v = random.random()
    while True:
        if v > icc(t, a, b, c[res - 1]):
            break
        res += 1
        if res >= K:
            break
    return res

def calcVar( L ):
    n = len(L)
    sum = 0.0
    for v in L:
        sum += v
    mean = sum / n
    ssum = 0.0
    for v in L:
        ssum += (mean - v) ** 2.0
    return ssum / (n - 1.0)

Nitems = 5                                 #    Number of items
X = [1, 1, 1, 1, 1]                        #    Responses to be set in simulation

a = [1.0, 1.0, 1.0, 1.0, 1.0]              #    Discrimination parameters
b = [-0.5, -0.3, 0.0, 0.3, 0.5]            #    Difficulty parameters
K = 2                                      #    Number of response categories
C = [0.0]                                  #    Category boundaries
#   K = 7
#   C = [-2.0, -1.0, -0.25, 0.25, 1.0, 2.0]

Data = {'K': K, 'Nitm': Nitems,  'C': C, 'a':a, 'b': b, 'X': X}

sm = StanModel(file = 'EstTheta.stan')
with open('model.pkl', 'wb') as f:
    pickle.dump(sm, f)

L_theta = []
L_precision = []
n_step = 1000
for step in range(n_step):
    if (step % 5 == 0):
        print("\nstep = {0}/{1}".format(step, n_step))
    t = random.gauss(0.0, 1.0)
    for i in range(Nitems):
        X[i] = CatResponse(t, a[i], b[i], C, K)       
        
    sm = pickle.load(open('model.pkl', 'rb'))
    fit = sm.sampling(data = Data,  n_jobs = 1)
    theta = fit['theta']
    var_theta = calcVar(theta)

    L_theta.append(t)
    L_precision.append(1.0 - var_theta)

sum = 0.0
for p in L_precision:
    sum += p
mean_precision = sum / len(L_precision)
 

plt.title("Mean precison = {0:>.3}".format(mean_precision)) 
plt.xlabel("theta")
plt.ylabel("precison") 
plt.axis([-4.0, 4.0, 0.0, 1.0])
plt.plot(L_theta, L_precision, 'bo') 
plt.show() 


